/*

  Amazonas
  --
  Jaime Vives Piqueres - May 2010
  
*/

//#version unofficial MegaPov 1.21;
#version 3.7;

#include "colors.inc"
#include "textures.inc"
#include "stones.inc"
#include "rand.inc"
#include "woods.inc"
#include "transforms.inc"

// ===========================================================
// radiosity control
// ===========================================================
#declare use_radiosity= 0; // 1=load, 2=save, 0=off
#declare rad_quality=   1; // 1-10 
#declare rad_bounces=   1; // >0 
#declare rad_brightness=1; // tune if needed
// ===========================================================
// other control switches:
// ===========================================================
#declare use_blur      =0; // 0=off, >0=on and amount*7 : focal blur switch
#declare use_area      =1; // 0=off, 1=on : focal blur switch
#declare use_water     =1; // 0=off, 1=on
#declare use_media     =1; // 0=off, 1=on
#declare use_forest    =1; // 0=off, 1=on
#declare use_clouds    =1; // 0=off, 1=on
#declare daylight      =1; // 0=off, 1=on : sunlight/sky switch

// ===========================================================
// global settings
// ===========================================================
global_settings {
 charset utf8
 max_trace_level 5   
 assumed_gamma 1.0   
 #if (use_radiosity) 
  radiosity {        
   #ifndef(rad_count) #declare rad_count=rad_quality; #end
   #if (use_radiosity=2)      // +RFsaved.rad +RFO
    // saving radiosity
    //save_file "saved.rad"
    pretrace_start 0.05
    pretrace_end 0.005
    count 160*rad_count
    nearest_count 10+rad_quality
    error_bound 0.5/rad_quality
    recursion_limit rad_bounces
    brightness rad_brightness
   #else                
    // loading radiosity      // +RFsaved.rad +RFI
    //load_file "saved.rad"
    pretrace_start 1
    pretrace_end 1
    count 1 //60*rad_count
    nearest_count 10+rad_quality
    error_bound 0.5/rad_quality
    recursion_limit rad_bounces
    always_sample off
    brightness rad_brightness
   #end
   //gray_threshold .5 
  }
 #end
}
// ===========================================================
// default finish for all textures:
#default{texture{finish{ambient 0 diffuse 1 conserve_energy}}}
// ===========================================================

#debug "BUILDING SKY DOME\n"

// ===========================================================
// LightSys: used to obtain light colors and fading, and also to provide a realisitic 
// sunlight and sky
// ===========================================================
// Include CIE color transformation macros by Ive : Lightsys is based on them
#include "CIE.inc" 
// change the CIE white point to typical film temperature
//CIE_ColorSystemWhitepoint(sRGB_ColSys, Illuminant_D55)
// Main lightsys include 
//#declare Lightsys_Scene_Scale=10;
#include "lightsys.inc" 
// Include predefined lumens, spectrums, kelvin temperatures 
#include "lightsys_constants.inc" 
// predefine some useful light colors from emissive spectrums 
#declare Cl_Daylight =Daylight(6500); 
#declare Cl_Skylight =Daylight(7500); 
// Brightness control (global light multiplier): 
#declare Lightsys_Brightness=1; // adjust for a brighter/dimmer image 
// ===========================================================
// sky sphere and sunlight from Lightsys
#if (daylight)                                    
 #declare North=-z; #declare Az=-90; #declare Al=60;
 #declare Intensity_Mult=2; // brightness of the sky dome and sunlight
 #declare Max_Vertices=1000;
 #include "CIE_Skylight"
 #declare Cl_Sunlight =Blackbody(2000+5500*Al/90); 
 light_source{
  SolarPosition
  Light_Color(Cl_Sunlight,SunColor.gray*pi*2)
  #if (use_area)
  area_light 40000*x,40000*z,8,8 jitter adaptive 1 orient circular
  #end                  
 } 
#end                   


#declare camera_location = <1, 10, -15>;
camera{
	 perspective
	 location camera_location
	 right (320 / 240) * x
	 up y
	 angle 47
	 look_at <0.5, 1, -4.5>
  #if (use_blur)
  aperture 2/3 blur_samples 7*use_blur focal_point <0, .1, 0>
  #end     
}

#debug "LOAD PIGMENTS\n"                         
#declare pgr1=pigment{image_map{jpeg "mayang_many_wet_leaves_2162814.jpg"} scale 1}                          
#declare pgr2=pigment{image_map{jpeg "mayang_roadside_plants.jpg"} scale 1}                          
#declare pgr3=pigment{image_map{jpeg "mayang_grass_6.jpg"} scale 1}                          
#declare pgr4=pigment{image_map{jpeg "long_thin_leaves_5132573.jpg"} scale 1}                          


// **************
// *** GROUND ***
// **************
#debug "TERRAIN CREATION\n"
#declare t_sand=
 texture{ 
  pigment_pattern{
   image_map{
    png "map_texture.png"
   }                    
  }
  texture_map{   
   [0 pigment{image_map{jpeg "sup060"}}]
   [1 pigment{
       granite 
       pigment_map{
       [0 pgr1]   
       [.33 pgr2]       
       [.67 pgr3]       
       [1 pgr4]       
       }
       turbulence .1 lambda 3
       scale .1
      }
    ]
  }
 }
#declare sand=
height_field{
 png "map_terrain.png"
 translate <-.5,0,-.5>
 texture{t_sand translate -.5 rotate 90*x}
 scale <20,0.25,20>
 translate .2*y
}
object{sand}


#debug "FILLING THE RIVER\n"
#if (use_water)
#if (use_radiosity=2)
 plane{y,0.385
   pigment{Gray}
 }
#else
 plane{y,0.385 hollow
   pigment{
   #if (use_media)
   rgbt 1
   #else
   Orange
   #end
   }    
   normal{
    bump_map{png "map_water.png" bump_size -1}   
    turbulence 1 lambda 4
    translate -.5               
    rotate 90*x
    scale 20
   }            
   finish {
        diffuse 0
        ambient 0
        reflection {0.02, 0.99 fresnel on}
        conserve_energy
        specular 0.4
        roughness 0.0035    
   }   
   interior{
    ior 1.33        
    fade_distance .01 fade_power 1001 fade_color (Orange*.75+Yellow*.25+Gray10)*.1
    #if (use_media)
    media{scattering{3,(Orange*.75+Yellow*.25+Gray10)*10}}
    #end
   }
 }
#end                        
#end                        
    

#if (use_forest)      
#macro rrand(Min, Max, RS) (floor(rand(RS)*(Max-Min+1) + Min )) #end
#declare f_distrib=
  function{
   pigment{
     bumps
     color_map{[0 rgb 0][1 rgb 1]}
   }
  }               
#declare f_sizes=
  function{
   pigment{
    image_map{png "map_forest.png"}
    rotate 90*x
    translate <-.5,0,-.5>
    scale 20
   }
  }                         


#declare ISOTREE=
union{
 isosurface{
  function{f_sphere(x,y,z,1) - f_granite(x*.2,y*.2,z*.2)*.9 - f_granite(x*.5,y*.5,z*.5)*.5 + f_granite(x*1,y*1,z*1)*.5}
  max_gradient 3
  contained_by{sphere{0,2}}
  double_illuminate
 }
 scale .2
}
#declare ISOTREE1=
object{ISOTREE
 pigment{pgr1 warp{spherical}}
}
#declare ISOTREE2=
object{ISOTREE
 pigment{pgr2 warp{spherical}}
}
#declare ISOTREE3=
object{ISOTREE
 pigment{pgr3 warp{spherical}}
}
#declare ISOTREE4=
object{ISOTREE
 pigment{pgr4 warp{spherical}}
}
#declare area_v=<20,0,20>;
#declare r_stone=seed(421);
#declare num_ram=30000;
#declare cnt=1;
#debug "PLANTING ISOTREES\n"
#while (cnt<=num_ram)
 #declare stonel=<-area_v.x*.5+area_v.x*rand(r_stone),0,-area_v.z*.5+area_v.z*rand(r_stone)>;
 #if (f_sizes(stonel.x,0,stonel.z).gray*.8<rand(r_stone))
  #declare stp=<stonel.x,1000,stonel.z>;
  #declare Norm=<0,0,0>;
  #declare Inter=trace(sand,stp,-y,Norm);
  #if (Norm.gray!=0) 
    #if (Inter.y>0.39)
    #declare wtree= int(rrand(1, 4, r_stone));
    object{              
     #switch (wtree)
      #case (1)
       ISOTREE1 
              rotate <360*rand(r_stone),360*rand(r_stone),360*rand(r_stone)>
       #break
      #case (2)
       ISOTREE2 
              rotate <360*rand(r_stone),360*rand(r_stone),360*rand(r_stone)>
       #break
      #case (3)
       ISOTREE3 
              rotate <360*rand(r_stone),360*rand(r_stone),360*rand(r_stone)>
       #break
      #case (4)
       ISOTREE4 
              rotate <360*rand(r_stone),360*rand(r_stone),360*rand(r_stone)>
       #break
      #else
     #end   
     scale .2+f_sizes(stonel.x,0,stonel.z).gray*.8
     scale (.75+.5*rand(r_stone))*<1,.75,1>*.75
     translate <stp.x,Inter.y,stp.z>
    }
    #end
  #end
  #declare cnt=cnt+1;
 #end
#end               
               
#debug "INCLUDE POVTREE1\n"
#include "tree1.inc"  
#debug "INCLUDE POVTREE2\n"
#include "tree2.inc"  
#debug "INCLUDE POVTREE3\n"
#include "tree3.inc"  
#debug "INCLUDE POVTREE4\n"
#include "tree4.inc"  
#declare num_ram=15000;
#declare cnt=1;
#debug "PLANTING POVTREES\n"
#while (cnt<=num_ram)
 #declare stonel=<-area_v.x*.5+area_v.x*rand(r_stone),0,-area_v.z*.5+area_v.z*rand(r_stone)>;
 #if (f_sizes(stonel.x,0,stonel.z).gray*.8<rand(r_stone))
  #declare stp=<stonel.x,1000,stonel.z>;
  #declare Norm=<0,0,0>;
  #declare Inter=trace(sand,stp,-y,Norm);
  #if (Norm.gray!=0) 
    #if (Inter.y>0.39)
    #declare wtree= int(rrand(1, 4, r_stone));
    object{              
     #switch (wtree)
      #case (1)
       POVTREE1 rotate <-5+10*rand(r_stone),360*rand(r_stone),0>
       translate -.5*(1-f_sizes(stonel.x,0,stonel.z).gray)*y
       #break
      #case (2)
       POVTREE2 rotate <-5+10*rand(r_stone),360*rand(r_stone),0>
       translate -.5*(1-f_sizes(stonel.x,0,stonel.z).gray)*y
       #break
      #case (3)
       POVTREE3 rotate <-5+10*rand(r_stone),360*rand(r_stone),0>
       translate -.5*(1-f_sizes(stonel.x,0,stonel.z).gray)*y
       #break
      #case (4)
       POVTREE4 rotate <-5+10*rand(r_stone),360*rand(r_stone),0>
       translate -.5*(1-f_sizes(stonel.x,0,stonel.z).gray)*y
       #break
      #else
     #end
     #if (rand(r_stone)>.25)   
     scale .2+f_sizes(stonel.x,0,stonel.z).gray*.8
     #end
     scale (.75+.25*rand(r_stone))*.75
     translate <stp.x,Inter.y,stp.z>
    }
    #end
  #end
  #declare cnt=cnt+1;
 #end
#end


#end

 
#if (use_clouds)
plane{
y,50 hollow
 pigment{
  leopard scale 2
  color_map{
   [0.3 rgbt 1]
   [0.5 rgbt .1]
  }            
  scale 1/10 warp{turbulence .5} scale 10
  warp{turbulence .3 lambda 3}
 }
}           
#end
  

